/*
 * Copyright 2006 Luther E. Birdzell.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.itko.lisa.ext.util;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;

/**
 * Helper methods for POI.
 * 
 * @author <a href="mailto:luther@dctmutils.org">Luther E. Birdzell</a>
 */
public class PoiHelper
{
    private static Logger log = Logger.getLogger(PoiHelper.class.getName());

    /**
     * Load the workbook file from the file system.
     * 
     * @param workbookPath
     * @return an <code>HSSFWorkbook</code>
     */
    public static HSSFWorkbook getWorkbook(String workbookPath)
    {
        HSSFWorkbook workbook = null;

        try
        {
            POIFSFileSystem fileSystem = getDocAsPoiFileSystem(workbookPath);
            workbook = new HSSFWorkbook(fileSystem);
        }
        catch ( IOException e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return workbook;
    }

    /**
     * Instantiate a <code>POIFSFileSystem</code> object for the specified file
     * located in the file system.
     * 
     * @param msOfficeFileName
     * @return an <code>POIFSFileSystem</code>
     */
    public static POIFSFileSystem getDocAsPoiFileSystem(String msOfficeFileName)
    {
        POIFSFileSystem fileSystem = null;

        try
        {
            if ( StringUtils.isBlank(msOfficeFileName) )
            {
                throw new RuntimeException("msOfficeFileName");
            }

            log.info("loading " + msOfficeFileName);
            InputStream input = new FileInputStream(msOfficeFileName);

            if ( input != null )
            {
                fileSystem = new POIFSFileSystem(input);
            }
            else
            {
                throw new RuntimeException("File : '" + msOfficeFileName
                        + "' not found.");
            }
        }
        catch ( IOException e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return fileSystem;
    }

    /**
     * Get the value of the specified cell as a <code>String</code>.
     * 
     * @param cell
     * @return - Value or empty <code>String</code> ("")
     */
    public static String getCellValueAsString(Cell cell)
    {
        String cellValue = "";

        if ( cell != null )

            switch ( cell.getCellType() )
            {

                case Cell.CELL_TYPE_BLANK:
                    break;
                case Cell.CELL_TYPE_BOOLEAN:
                    cellValue = String.valueOf(cell.getBooleanCellValue());
                    break;
                case Cell.CELL_TYPE_ERROR:
                    cellValue = String.valueOf(cell.getErrorCellValue());
                    break;
                case Cell.CELL_TYPE_FORMULA:
                    cellValue = cell.getCellFormula();
                    break;
                case Cell.CELL_TYPE_NUMERIC:
                    cellValue = String.valueOf(cell.getNumericCellValue());
                    cellValue = StringUtils.substring(cellValue,
                                                      0,
                                                      StringUtils.indexOf(cellValue,
                                                                          "."));
                    break;
                case Cell.CELL_TYPE_STRING:
                    cellValue = cell.getRichStringCellValue().getString();
                    break;
                default:
                    throw new RuntimeException("Unsupported Cell Type");
            }

        return StringUtils.trim(cellValue);
    }

    /**
     * Dump the contents of the specified spread sheet.
     * 
     * @param sheetName
     */
    public static void dumpSpreadSheet(String sheetName)
    {
        HSSFWorkbook wb = PoiHelper.getWorkbook(sheetName);
        HSSFSheet sheet = wb.getSheetAt(0);

        // Iterate over each row in the sheet
        Iterator < Row > rows = sheet.rowIterator();

        while ( rows.hasNext() )
        {
            Row row = rows.next();
            log.fine("Row #" + row.getRowNum());

            // Iterate over each cell in the row and print out the cell's
            // content
            Iterator < Cell > cells = row.cellIterator();
            while ( cells.hasNext() )
            {
                Cell cell = cells.next();
                log.fine("Cell #" + cell.getRowIndex());
                switch ( cell.getCellType() )
                {

                    case Cell.CELL_TYPE_BLANK:
                        break;
                    case Cell.CELL_TYPE_BOOLEAN:
                        log.fine("" + cell.getBooleanCellValue());
                        break;
                    case Cell.CELL_TYPE_ERROR:
                        log.fine("" + cell.getErrorCellValue());
                        break;
                    case Cell.CELL_TYPE_FORMULA:
                        log.fine("" + cell.getCellFormula());
                        break;
                    case Cell.CELL_TYPE_NUMERIC:
                        log.fine("" + cell.getNumericCellValue());
                        break;
                    case Cell.CELL_TYPE_STRING:
                        log.fine(cell.getRichStringCellValue().getString());
                        break;
                    default:
                        log.fine("unsuported sell type");
                        break;
                }
            }
        }
    }

    /**
     * 
     * 
     * @param workbookName
     * @param worksheetName
     * @return the number of rows in the specified worksheet.
     */
    public static int getNumberOfRowsInWorksheet(String workbookName,
                                                 String worksheetName)
    {
        int numberOfDataRows = 0;

        try
        {
            HSSFWorkbook workbook = getWorkbook(workbookName);
            HSSFSheet worksheet = workbook.getSheet(worksheetName);

            // Iterate over each row in the sheet
            Iterator < Row > rows = worksheet.rowIterator();

            while ( rows.hasNext() )
            {
                Row row = rows.next();
                Cell cell = row.getCell(( short ) 0);
                String cellValue = getCellValueAsString(cell);

                if ( StringUtils.isNotBlank(cellValue) )
                {
                    numberOfDataRows++;
                }
            }
        }
        catch ( Exception e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return numberOfDataRows;
    }

    /**
     * 
     * 
     * @param cell
     * @param value
     */
    public static void setCellValue(Cell cell, String value)
    {
        if ( cell != null && StringUtils.isNotBlank(value) )
        {
            cell.setCellValue(new HSSFRichTextString(value));
        }
    }

    /**
     * 
     * 
     * @param reportOutputPath
     * @param workbook
     */
    public static void writeWorkbookToFileSystem(String reportOutputPath,
                                                 HSSFWorkbook workbook)
    {
        if ( StringUtils.isNotBlank(reportOutputPath) && workbook != null )
        {
            FileOutputStream fos = null;

            try
            {
                fos = new FileOutputStream(reportOutputPath);
                workbook.write(fos);
            }
            catch ( IOException e )
            {
                log.log(Level.SEVERE, e.getMessage(), e);
                throw new RuntimeException(e);
            }
            finally
            {
                IOUtils.closeQuietly(fos);
            }
        }
    }
}
